import urllib.request
import struct

def rc(x):
    start = 0
    for c in x:
        start ^= c
    return start.to_bytes(1, 'little')
    
def serial_keys(keys):
    token = b'\x00\x05\x00\x00\x00\x00\x01'
    total_len = len(token) + len(keys) + 3
    data = bytes([total_len]) + token + bytes([len(keys)]) + keys.encode('utf-8')
    return data + rc(data)

def key_from_serial(data):
    token = b'\x00\x05\x00\x00\x00\x00\x01'
    total_len = data[0]
    key_len = total_len - len(token) - 3
    keys = data[-1 - key_len:-1].decode('utf-8')
    return keys

def open_sogou(keys, durtot=0, version='3.7'):
    url = 'http://shouji.sogou.com/web_ime/mobile.php?durtot={}&h=000000000000000&r=store_mf_wandoujia&v={}'.format(durtot, version)
    data = serial_keys(keys)
    return urllib.request.urlopen(url, data)

def parse_result(result):
    words = []
    if result[0] + 2 != len(result):
        print('Error: invalid size')
        return words

    num_words = struct.unpack('<H', result[0x12:0x12+2])[0]
    if num_words == 0 or num_words > 32:
        print('Warning: strange words num', num_words)

    print(num_words)
    pos = 0x14
    for i in range(num_words):
        str_len = struct.unpack('<H', result[pos:pos+2])[0]
        if str_len == 0 or str_len > 0xFF:
            raise ValueError(result)            
        pos += 2
        if str_len == 0:
            continue
        word = result[pos:pos+str_len].decode('utf-16-le')
        try:
            word.encode('gb18030')
        except:
            print('Warning: word {} cant encode to gb18030'.format(repr(word)))

        words.append(word)
        pos += str_len
        str_len = struct.unpack('<H', result[pos:pos+2])[0]
        pos += str_len + 2
        str_len = struct.unpack('<H', result[pos:pos+2])[0]
        pos += str_len + 2 + 1

    if pos != len(result):
        print('Warning: buffer not exhausted!')
        
    return words
    
def get_cloud_words(keys):
    res = open_sogou(keys)
    if res.status != 200:
        print('Error: invalid response for input <{}>'.format(keys))
        return []

    result = res.read()
    return parse_result(result)

if __name__ == "__main__":
    matched_list = get_cloud_words('yuanmengzhixing')
    if matched_list:
        for matched_word in matched_list:
            print(matched_word)